# Generated by Django 4.2.19 on 2025-02-12 14:19

import NEMO.fields
import NEMO.utilities
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import NEMO_custom_forms.pdf_utils


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomForm",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("form_number", models.CharField(blank=True, max_length=255, null=True, unique=True)),
                (
                    "creation_time",
                    models.DateTimeField(auto_now_add=True, help_text="The date and time when the form was created."),
                ),
                (
                    "last_updated",
                    models.DateTimeField(auto_now=True, help_text="The last time this form was modified."),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[(0, "Pending"), (1, "Approved"), (2, "Rejected"), (3, "Closed")], default=0
                    ),
                ),
                ("template_data", models.TextField(blank=True, null=True)),
                ("notes", models.TextField(blank=True, null=True)),
                ("cancelled", models.BooleanField(default=False, help_text="Indicates the form has been cancelled.")),
                ("cancellation_time", models.DateTimeField(blank=True, null=True)),
                ("cancellation_reason", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "cancelled_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="custom_forms_cancelled",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="custom_forms_created",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "last_updated_by",
                    models.ForeignKey(
                        blank=True,
                        help_text="The last user who modified this form.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="custom_forms_updated",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "ordering": ["-last_updated"],
            },
        ),
        migrations.CreateModel(
            name="CustomFormPDFTemplate",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("enabled", models.BooleanField(default=True)),
                (
                    "name",
                    models.CharField(help_text="The unique name for this form template", max_length=255, unique=True),
                ),
                (
                    "create_permissions",
                    NEMO.fields.MultiRoleGroupPermissionChoiceField(
                        help_text="The roles/groups required for users to submit this type of custom form"
                    ),
                ),
                (
                    "view_all_permissions",
                    NEMO.fields.MultiRoleGroupPermissionChoiceField(
                        help_text="The roles/groups required for users to view all custom forms of this type. Users can always see custom forms they have submitted"
                    ),
                ),
                (
                    "form",
                    models.FileField(
                        help_text="The pdf form",
                        upload_to=NEMO.utilities.document_filename_upload,
                        validators=[NEMO_custom_forms.pdf_utils.validate_pdf_form],
                    ),
                ),
                ("form_fields", models.TextField(help_text="JSON formatted fields list")),
                (
                    "filename_template",
                    models.CharField(
                        default="{{ form.form_number|default_if_none:'' }}-{{ form.creation_time|date:'Y-md' }}",
                        help_text='<div style="margin-top: 10px">The filename template for the generated PDF. Provided variables include: <ul style="margin-left: 20px"><li style="list-style: inherit"><b>form</b>: the custom form instance</li><li style="list-style: inherit"><b>form_data</b>: a dictionary containing all dynamic fields and their values</li></ul></div>',
                        max_length=1024,
                    ),
                ),
            ],
            options={
                "ordering": ["name"],
            },
        ),
        migrations.CreateModel(
            name="CustomFormDocumentType",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(help_text="The unique name for this item", max_length=255, unique=True)),
                (
                    "display_order",
                    models.IntegerField(
                        help_text="The display order is used to sort these items. The lowest value category is displayed first."
                    ),
                ),
                (
                    "form_template",
                    models.ForeignKey(
                        blank=True,
                        help_text="Select a template this document type applies to. Leave blank for all.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="NEMO_custom_forms.customformpdftemplate",
                    ),
                ),
            ],
            options={
                "ordering": ["display_order", "name"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="CustomFormDocuments",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "document",
                    models.FileField(
                        blank=True,
                        max_length=255,
                        null=True,
                        upload_to=NEMO.utilities.document_filename_upload,
                        verbose_name="Document",
                    ),
                ),
                ("url", models.URLField(blank=True, null=True, verbose_name="URL")),
                (
                    "name",
                    models.CharField(
                        blank=True,
                        help_text="The optional name to display for this document",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "display_order",
                    models.IntegerField(
                        default=1,
                        help_text="The order in which choices are displayed on the landing page, from left to right, top to bottom. Lower values are displayed first.",
                    ),
                ),
                ("uploaded_at", models.DateTimeField(auto_now_add=True)),
                (
                    "custom_form",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="NEMO_custom_forms.customform"),
                ),
                (
                    "document_type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="NEMO_custom_forms.customformdocumenttype",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Custom form documents",
                "ordering": ["display_order", "document_type__display_order"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="CustomFormDisplayColumn",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "field_name",
                    models.CharField(
                        help_text="The pdf template field name whose value will be displayed in the table",
                        max_length=255,
                    ),
                ),
                ("display_order", models.PositiveIntegerField(help_text="The display order of the column.")),
                (
                    "display_name",
                    models.CharField(
                        blank=True,
                        help_text="The column name to be displayed in the table. If not provided, the field name will be used",
                        max_length=100,
                        null=True,
                    ),
                ),
                (
                    "template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="NEMO_custom_forms.customformpdftemplate"
                    ),
                ),
            ],
            options={
                "ordering": ["display_order"],
            },
        ),
        migrations.CreateModel(
            name="CustomFormAutomaticNumbering",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "enabled",
                    models.BooleanField(
                        default=True,
                        help_text="Uncheck this box to disable the automatic numbering sequence for this template",
                    ),
                ),
                (
                    "numbering_group",
                    models.IntegerField(
                        blank=True,
                        help_text="Optional. Use this field (with the year number for example) to create separate numbering sequences when its value is changed.",
                        null=True,
                    ),
                ),
                (
                    "numbering_per_user",
                    models.BooleanField(
                        default=False,
                        help_text="Check this box to maintain a separate numbering sequence for each user who generates it.",
                    ),
                ),
                (
                    "numbering_template",
                    models.CharField(
                        default="{{ current_number|stringformat:'04d' }}",
                        help_text='<div style="margin-top: 10px">The numbering template. Provided variables include: <ul style="margin-left: 20px"><li style="list-style: inherit"><b>custom_form_template</b>: the custom form template instance</li><li style="list-style: inherit"><b>user</b>: the user triggering the sequence</li><li style="list-style: inherit"><b>numbering_group</b>: the numbering group for this sequence</li><li style="list-style: inherit"><b>current_number</b>: the current number for this sequence</li></ul></div>',
                        max_length=1024,
                    ),
                ),
                (
                    "role",
                    NEMO.fields.RoleGroupPermissionChoiceField(
                        blank=True,
                        help_text="The role/group required for users to automatically generate the form number. Leave blank to generate it upon creation without user action.",
                        null=True,
                        verbose_name="Role/Group",
                    ),
                ),
                (
                    "template",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE, to="NEMO_custom_forms.customformpdftemplate"
                    ),
                ),
            ],
            options={
                "ordering": ["template__name"],
            },
        ),
        migrations.CreateModel(
            name="CustomFormAction",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "action_type",
                    NEMO.fields.DynamicChoicesCharField(
                        default="approval", help_text="The action type", max_length=100
                    ),
                ),
                ("name", models.CharField(blank=True, help_text="The optional action name", max_length=100, null=True)),
                (
                    "rank",
                    models.PositiveIntegerField(
                        help_text="The action rank number. Actions will be requested in ascending order"
                    ),
                ),
                (
                    "role",
                    NEMO.fields.RoleGroupPermissionChoiceField(
                        help_text="The role/group required for users to take the action", verbose_name="Role/Group"
                    ),
                ),
                (
                    "notification_email",
                    NEMO.fields.MultiEmailField(
                        blank=True,
                        help_text="Email addresses to cc when this action is taken",
                        max_length=2000,
                        null=True,
                    ),
                ),
                (
                    "self_action_allowed",
                    models.BooleanField(
                        default=False,
                        help_text="Check this box to allow the user who creates the form to also take action on it (approve his own form for example).",
                    ),
                ),
                (
                    "can_edit_form",
                    models.BooleanField(
                        default=True, help_text="Check this box if the candidate can make changes to the form"
                    ),
                ),
                (
                    "template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="NEMO_custom_forms.customformpdftemplate"
                    ),
                ),
            ],
            options={
                "ordering": ["template", "rank"],
                "unique_together": {("template", "rank")},
            },
        ),
        migrations.AddField(
            model_name="customform",
            name="template",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="NEMO_custom_forms.customformpdftemplate"
            ),
        ),
        migrations.CreateModel(
            name="CustomFormSpecialMapping",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "field_name",
                    models.CharField(help_text="The pdf template field name to map this value to", max_length=255),
                ),
                (
                    "field_value",
                    NEMO.fields.DynamicChoicesCharField(help_text="The special value to map it to", max_length=255),
                ),
                (
                    "field_value_boolean",
                    models.CharField(
                        blank=True,
                        help_text="Comma separated values to map approved/denied states, i.e. 'Yes,No'",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "field_value_action",
                    models.ForeignKey(
                        blank=True,
                        help_text="The action (for action mappings only)",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="NEMO_custom_forms.customformaction",
                    ),
                ),
                (
                    "template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="NEMO_custom_forms.customformpdftemplate"
                    ),
                ),
            ],
            options={
                "ordering": ["field_name"],
                "unique_together": {("template", "field_name")},
            },
        ),
        migrations.CreateModel(
            name="CustomFormActionRecord",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "action_type",
                    NEMO.fields.DynamicChoicesCharField(
                        default="approval", help_text="The action type", max_length=100
                    ),
                ),
                ("action_rank", models.PositiveIntegerField(help_text="The action rank number.")),
                (
                    "action_time",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time when the action was taken on this form."
                    ),
                ),
                ("action_result", models.BooleanField(help_text="Whether the action result was positive or negative")),
                (
                    "action_taken_by",
                    models.ForeignKey(
                        help_text="The user who took the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="custom_forms_reviewed",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "custom_form",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="NEMO_custom_forms.customform"),
                ),
            ],
            options={
                "ordering": ["-action_time"],
                "unique_together": {("custom_form", "action_type", "action_rank")},
            },
        ),
    ]
